
local FOLDER_NAME, SSA = ...

local L = LibStub("AceLocale-3.0"):GetLocale("ShamanAuras", true)
local Auras = LibStub("AceAddon-3.0"):GetAddon("ShamanAuras")
local AceGUI = LibStub("AceGUI-3.0");

local lgIcon,lgGlow,smIcon,smGlow = 32,45,25,35;
local x,y = 0,0,0;
local bg,low,med,high = "BACKGROUND","LOW","MEDIUM","HIGH"

-- Aura Group Builder
local function CreateGroup(name,parent)
	SSA[name] = CreateFrame("Frame",name,parent);
	local Group = SSA[name];
	Group:SetFrameStrata(bg);
	
	return Group;
end

-- Aura Icon Builder
local function InitializeFrames(name,parent,icon,iconSize,glowSize,charge)
	SSA[name] = CreateFrame("Frame",name,parent);
	local Frame = SSA[name];
	Frame:SetFrameStrata("LOW");
	Frame:SetWidth(iconSize);
	Frame:SetHeight(iconSize);

	Frame.texture = Frame:CreateTexture(nil,"BACKGROUND");
	Frame.texture:SetTexture("Interface\\addons\\ShamanAuras\\media\\ICONS\\"..icon);
	Frame.texture:SetAllPoints(Frame);
	
	Frame.CD = CreateFrame("Cooldown", name.."CD", Frame, "CooldownFrameTemplate");
	Frame.CD:SetAllPoints(Frame);
	
	Frame.CD.text = Frame.CD:CreateFontString(nil, "MEDIUM", "GameFontHighlightLarge");
	Frame.CD.text:SetAllPoints(Frame.CD);
	Frame.CD.text:SetPoint("CENTER",0,0);
	Frame.CD.text:SetTextColor(1,1,0,1);
	
	if (glowSize) then
		Frame.glow = CreateFrame("Frame",name.."Glow",Frame);
		Frame.glow:SetPoint("CENTER",0,0);
		Frame.glow:SetFrameStrata("BACKGROUND");
		Frame.glow:SetWidth(glowSize);
		Frame.glow:SetHeight(glowSize);
		Frame.glow:Show();
	end
	if (charge) then
		Frame.ChargeCD = CreateFrame("Cooldown", name.."ChargeCD", Frame, "CooldownFrameTemplate");
		Frame.ChargeCD:SetAllPoints(Frame);
		Frame.ChargeCD:SetFrameStrata("LOW");
		Frame.ChargeCD:Show()

		Frame.Charges = CreateFrame("Frame",name.."Charges",Frame);
		Frame.Charges:SetAllPoints(Frame);
		Frame.Charges:SetFrameStrata(med);
		Frame.Charges:SetWidth(iconSize);
		Frame.Charges:SetHeight(iconSize);

		Frame.Charges.text = Frame.Charges:CreateFontString(nil, "MEDIUM", "GameFontHighlightLarge");
		Frame.Charges.text:SetPoint("BOTTOMRIGHT",-3,3);
		Frame.Charges.text:SetFont("Interface\\addons\\ShamanAuras\\Media\\fonts\\ABF.ttf", 13.5,"OUTLINE");
		Frame.Charges.text:SetTextColor(1,1,1,1);	
	end
end


-----------------------------------------------------------------------
-- Enhancement Frames
-----------------------------------------------------------------------
local AuraGroup,LargeIconGrpTop,LargeIconGrpBot,SmallIconGrpLeft,SmallIconGrpRight

-- Build Enhtoration Aura Group Containers
AuraGroup = CreateGroup("AuraGroupEnh",UIParent);
LargeIconGrpTop = CreateGroup("LargeIconGrpTopEnh",AuraGroup);
LargeIconGrpBot = CreateGroup("LargeIconGrpBotEnh",AuraGroup);
SmallIconGrpLeft = CreateGroup("SmallIconGrpLeftEnh",AuraGroup);
SmallIconGrpRight = CreateGroup("SmallIconGrpRightEnh",AuraGroup);

-- Build Large Enhancement Icon Frames
InitializeFrames("AscendanceEnh",LargeIconGrpBot,"shared\\ascendance",lgIcon);
InitializeFrames("Boulderfist",LargeIconGrpBot,"enhancement\\boulderfist",lgIcon,lgGlow,true);
InitializeFrames("CrashLightning",LargeIconGrpTop,"enhancement\\crash_lightning",lgIcon,lgGlow);
InitializeFrames("DoomWinds",LargeIconGrpBot,"enhancement\\doom_winds",lgIcon);
InitializeFrames("EarthenSpike",LargeIconGrpBot,"enhancement\\earthen_spike",lgIcon);
InitializeFrames("FeralSpirit",LargeIconGrpBot,"enhancement\\feral_spirit",lgIcon);
InitializeFrames("Flametongue",LargeIconGrpTop,"enhancement\\flametongue",lgIcon,lgGlow);
InitializeFrames("Frostbrand",LargeIconGrpTop,"enhancement\\frostbrand",lgIcon,lgGlow);
InitializeFrames("Stormstrike",LargeIconGrpTop,"enhancement\\stormstrike",lgIcon,lgGlow,true);
InitializeFrames("Sundering",LargeIconGrpBot,"enhancement\\sundering",lgIcon);
InitializeFrames("Windsong",LargeIconGrpBot,"enhancement\\windsong",lgIcon,lgGlow);

InitializeFrames("AstralShiftEnh",SmallIconGrpRight,"shared\\astral_shift",smIcon);
InitializeFrames("CleanseSpiritEnh",SmallIconGrpLeft,"restoration\\purify_spirit",smIcon);
InitializeFrames("EarthgrabTotemEnh",SmallIconGrpRight,"totems\\earthgrab_totem",smIcon);
InitializeFrames("FeralLunge",SmallIconGrpRight,"enhancement\\feral_lunge",smIcon);
InitializeFrames("HexEnh",SmallIconGrpLeft,"shared\\hex",smIcon);
InitializeFrames("LightningSurgeTotemEnh",SmallIconGrpRight,"totems\\lightning_surge_totem",smIcon);
InitializeFrames("Rainfall",SmallIconGrpLeft,"shared\\healing_rain",smIcon);
InitializeFrames("SpiritWalk",SmallIconGrpRight,"enhancement\\spirit_walk",smIcon);
InitializeFrames("VoodooTotemEnh",SmallIconGrpLeft,"totems\\voodoo_totem",smIcon);
InitializeFrames("WindRushTotemEnh",SmallIconGrpRight,"totems\\wind_rush_totem",smIcon);
InitializeFrames("WindShearEnh",SmallIconGrpLeft,"shared\\wind_shear",smIcon);

-------------------------------------------------------------------------------------------------------
----- Build and Initialize Status Bars
-------------------------------------------------------------------------------------------------------

-- Maelstrom Bar
SSA.MaelstromBarEnh = CreateFrame("StatusBar","MaelstromBarEnh",AuraGroup);
local MaelstromBar = SSA.MaelstromBarEnh;

MaelstromBar:SetStatusBarTexture("Interface\\addons\\ShamanAuras\\media\\statusbar\\fifths");
MaelstromBar:GetStatusBarTexture():SetHorizTile(false);
MaelstromBar:GetStatusBarTexture():SetVertTile(false);
MaelstromBar:SetPoint("CENTER",AuraGroup,"CENTER",0,-139);
MaelstromBar:SetWidth(260);
MaelstromBar:SetHeight(21);
MaelstromBar:SetFrameStrata("LOW");
MaelstromBar:SetStatusBarColor(0,0.5,1);
MaelstromBar:SetAlpha(0);

MaelstromBar.bg = MaelstromBar:CreateTexture(nil,"BACKGROUND");
MaelstromBar.bg:SetTexture("Interface\\TargetingFrame\\UI-StatusBar");
MaelstromBar.bg:SetAllPoints(true);
MaelstromBar.bg:SetVertexColor(0,0,0);
MaelstromBar.bg:SetAlpha(0.5);

MaelstromBar.text = MaelstromBar:CreateFontString(nil, "HIGH", "GameFontHighlightLarge");
MaelstromBar.text:SetPoint("CENTER",MaelstromBar,"CENTER",0,0);
MaelstromBar.text:SetFont("Fonts\\FRIZQT__.TTF", 12);
MaelstromBar.text:SetTextColor(1,1,1,1);

MaelstromBar.Lightning = CreateFrame("PlayerModel","MaelstromBarEnhLightning",MaelstromBar);
MaelstromBar.Lightning:SetModel("SPELLS/LIGHTNING_AREA_DISC_STATE.m2");
MaelstromBar.Lightning:SetFrameStrata("MEDIUM");
MaelstromBar.Lightning:SetPosition(0,0,-2);
MaelstromBar.Lightning:SetAllPoints(MaelstromBar);
MaelstromBar.Lightning:SetAlpha(0);

SSA.BuffTimerBarGrpEnh = CreateFrame("Frame","BuffTimerBarGrpEnh",AuraGroup);
local BuffTimerBarGrp = SSA.BuffTimerBarGrpEnh;
BuffTimerBarGrp:SetWidth(131);
BuffTimerBarGrp:SetHeight(180);
BuffTimerBarGrp:SetBackdrop(nil);
BuffTimerBarGrp:Show();

SSA.MainTimerBarGrpEnh = CreateFrame("Frame","MainTimerBarGrpEnh",AuraGroup);
local MainTimerBarGrp = SSA.MainTimerBarGrpEnh;
MainTimerBarGrp:SetWidth(131);
MainTimerBarGrp:SetHeight(180);
BuffTimerBarGrp:SetBackdrop(nil);
MainTimerBarGrp:Show();

SSA.UtilTimerBarGrpEnh = CreateFrame("Frame","UtilTimerBarGrpEnh",AuraGroup);
local UtilTimerBarGrp = SSA.UtilTimerBarGrpEnh;
UtilTimerBarGrp:SetWidth(47);
UtilTimerBarGrp:SetHeight(180);
UtilTimerBarGrp:SetBackdrop(nil);
UtilTimerBarGrp:Show();

SSA.AscendanceBarEnh = CreateFrame("StatusBar","AscendanceBarEnh",BuffTimerBarGrp);
SSA.AstralShiftBarEnh = CreateFrame("StatusBar","AstralShiftBarEnh",BuffTimerBarGrp);
--SSA.BloodlustBarEnh = CreateFrame("StatusBar","BloodlustBarEnh",BuffTimerBarGrp);
SSA.BoulderfistBar = CreateFrame("StatusBar","BoulderfistBar",MainTimerBarGrp);
SSA.CrashLightningBar = CreateFrame("StatusBar","CrashLightningBar",MainTimerBarGrp);
SSA.EarthgrabTotemBarEnh = CreateFrame("StatusBar","EarthgrabTotemBarEnh",UtilTimerBarGrp);
SSA.EarthenSpikeBar = CreateFrame("StatusBar","EarthenSpikeBar",UtilTimerBarGrp);
SSA.FlametongueBar = CreateFrame("StatusBar","FlametongueBar",MainTimerBarGrp);
SSA.FrostbrandBar = CreateFrame("StatusBar","FrostbrandBar",MainTimerBarGrp);
SSA.HeroismBarEnh = CreateFrame("StatusBar","HeroismBarEnh",BuffTimerBarGrp);
SSA.HexBarEnh = CreateFrame("StatusBar","HexBarEnh",UtilTimerBarGrp);
SSA.SpiritWalkBar = CreateFrame("StatusBar","SpiritWalkBar",BuffTimerBarGrp);
SSA.TimeWarpBarEnh = CreateFrame("StatusBar","TimeWarpBarEnh",BuffTimerBarGrp);
SSA.VoodooTotemBarEnh = CreateFrame("StatusBar","VoodooTotemBarEnh",UtilTimerBarGrp);
SSA.WindRushTotemBarEnh = CreateFrame("StatusBar","WindRushTotemBarEnh",UtilTimerBarGrp);
SSA.WindsongBar = CreateFrame("StatusBar","WindsongBar",MainTimerBarGrp);

-- Doom Winds Alert Texture
SSA.DoomWindsTex = CreateFrame("Frame","DoomWindsTex",AuraGroup);
local DoomWindsTex = SSA.DoomWindsTex;
DoomWindsTex:SetPoint("CENTER",AuraGroup,"CENTER",-150,0);
DoomWindsTex:SetWidth(125);
DoomWindsTex:SetHeight(240);
DoomWindsTex:SetFrameStrata("BACKGROUND");
--DoomWindsTex:SetStatusBarColor(1,1,1);
DoomWindsTex:Hide();

DoomWindsTex.texture = DoomWindsTex:CreateTexture(nil,"LOW");
DoomWindsTex.texture:SetTexture("Textures\\SpellActivationOverlays\\Dark_Tiger");
DoomWindsTex.texture:SetPoint("CENTER",DoomWindsTex,"CENTER",0,0);
DoomWindsTex.texture:SetWidth(DoomWindsTex:GetWidth() + 20);
DoomWindsTex.texture:SetHeight(DoomWindsTex:GetHeight());
--DoomWindsTex.texture:SetRotation(math.rad(-90));
DoomWindsTex.texture:SetAlpha(1);

DoomWindsTex.Scaler = DoomWindsTex:CreateAnimationGroup();
DoomWindsTex.Scaler:SetLooping("REPEAT");

local scalePush = 1.08;
local scalePull = 1 / scalePush;

DoomWindsTex.Scaler.scaleOut = DoomWindsTex.Scaler:CreateAnimation("Scale")
DoomWindsTex.Scaler.scaleOut:SetOrder(1)
DoomWindsTex.Scaler.scaleOut:SetDuration(0.5)
DoomWindsTex.Scaler.scaleOut:SetSmoothing("IN_OUT")
DoomWindsTex.Scaler.scaleOut:SetFromScale(1,1)
DoomWindsTex.Scaler.scaleOut:SetToScale(1,scalePush)
 
DoomWindsTex.Scaler.scaleIn = DoomWindsTex.Scaler:CreateAnimation("Scale")
DoomWindsTex.Scaler.scaleIn:SetOrder(2)
DoomWindsTex.Scaler.scaleIn:SetDuration(0.5)
DoomWindsTex.Scaler.scaleIn:SetSmoothing("IN_OUT")
DoomWindsTex.Scaler.scaleIn:SetFromScale(1,1)
DoomWindsTex.Scaler.scaleIn:SetToScale(1,scalePull)

-- Stormstrike Charges
local elapsed
SSA.StormstrikeChargeGrp = CreateFrame("Frame","StormstrikeChargeGrp",AuraGroup);
local StormstrikeCharges = SSA.StormstrikeChargeGrp;
StormstrikeCharges:SetFrameStrata(bg);
StormstrikeCharges:SetWidth(260);
StormstrikeCharges:SetHeight(50);
StormstrikeCharges:SetPoint("CENTER",0,-100);
StormstrikeCharges:Show();

StormstrikeCharges.Charge1 = CreateFrame("Frame","StormstrikeCharge1",StormstrikeChargeGrp);
StormstrikeCharges.Charge1:SetWidth(100);
StormstrikeCharges.Charge1:SetHeight(100);
StormstrikeCharges.Charge1:SetPoint("LEFT",StormstrikeChargeGrp,"LEFT",15,-3);
StormstrikeCharges.Charge1:SetAlpha(0);
StormstrikeCharges.Charge1:Show();

StormstrikeCharges.Charge1.Lightning = CreateFrame("PlayerModel","StormstrikeCharge1_Lightning",StormstrikeCharges.Charge1);
StormstrikeCharges.Charge1.Lightning:SetModel("spells/Monk_chiblast_precast.m2");
StormstrikeCharges.Charge1.Lightning:SetFrameStrata(low);
StormstrikeCharges.Charge1.Lightning:SetPosition(0,0,0);
StormstrikeCharges.Charge1.Lightning:SetAllPoints(StormstrikeCharges.Charge1);
StormstrikeCharges.Charge1.Lightning:SetAlpha(1);
elapsed = 0;
StormstrikeCharges.Charge1.Lightning:HookScript("OnUpdate", function(self, elaps)
	elapsed = elapsed + (elaps * 1000);
	self:SetSequenceTime(1, elapsed);
end);

StormstrikeCharges.Charge2 = CreateFrame("Frame","StormstrikeCharge2",StormstrikeChargeGrp);
StormstrikeCharges.Charge2:SetWidth(100);
StormstrikeCharges.Charge2:SetHeight(100);
StormstrikeCharges.Charge2:SetPoint("RIGHT",StormstrikeChargeGrp,"RIGHT",-15,-3);
StormstrikeCharges.Charge2:SetAlpha(0);
StormstrikeCharges.Charge2:Show();

StormstrikeCharges.Charge2.Lightning = CreateFrame("PlayerModel","StormstrikeCharge2_Lightning",StormstrikeCharges.Charge2);
StormstrikeCharges.Charge2.Lightning:SetModel("spells/Monk_chiblast_precast.m2");
StormstrikeCharges.Charge2.Lightning:SetFrameStrata(low);
StormstrikeCharges.Charge2.Lightning:SetPosition(0,0,0);
StormstrikeCharges.Charge2.Lightning:SetAllPoints(StormstrikeCharges.Charge2);
StormstrikeCharges.Charge2.Lightning:SetAlpha(1);
elapsed = 0;
StormstrikeCharges.Charge2.Lightning:HookScript("OnUpdate", function(self, elaps)
	elapsed = elapsed + (elaps * 1000);
	self:SetSequenceTime(1, elapsed);
end);

StormstrikeCharges.Charge3 = CreateFrame("Frame","StormstrikeCharge3",StormstrikeChargeGrp);
StormstrikeCharges.Charge3:SetWidth(100);
StormstrikeCharges.Charge3:SetHeight(100);
StormstrikeCharges.Charge3:SetPoint("CENTER",StormstrikeChargeGrp,"CENTER",0,-3);
StormstrikeCharges.Charge3:SetAlpha(0);
StormstrikeCharges.Charge3:Show();

StormstrikeCharges.Charge3.Lightning = CreateFrame("PlayerModel","StormstrikeCharge3_Lightning",StormstrikeCharges.Charge3);
StormstrikeCharges.Charge3.Lightning:SetModel("spells/Monk_chiblast_precast.m2");
StormstrikeCharges.Charge3.Lightning:SetFrameStrata(low);
StormstrikeCharges.Charge3.Lightning:SetPosition(0,0,0);
StormstrikeCharges.Charge3.Lightning:SetAllPoints(StormstrikeCharges.Charge3);
StormstrikeCharges.Charge3.Lightning:SetAlpha(1);
elapsed = 0;
StormstrikeCharges.Charge3.Lightning:HookScript("OnUpdate", function(self, elaps)
	elapsed = elapsed + (elaps * 1000);
	self:SetSequenceTime(1, elapsed);
end);

-------------------------------------------------------------------------------------------------------
----- Build Move UI
-------------------------------------------------------------------------------------------------------

-- Initialize Check Button Frames
SSA.MoveEnh = CreateFrame("Frame","MoveEnh",UIParent);

SSA.AuraObjectsEnh = {
	[1] = {
		alpha = nil,
		object = AuraGroup,
		backdrop = 'BackdropSB',
	},	
	[2] = {
		alpha = nil,
		object = LargeIconGrpTop,
		backdrop = 'BackdropSB',
	},
	[3] = {
		alpha = nil,
		object = LargeIconGrpBot,
		backdrop = 'BackdropSB',
	},
	[4] = {
		alpha = nil,
		object = SmallIconGrpLeft,
		backdrop = 'BackdropSB',
	},
	[5] = {
		alpha = nil,
		object = SmallIconGrpRight,
		backdrop = 'BackdropSB',
	},
	[6] = {
		alpha = nil,
		object = BuffTimerBarGrp,
		backdrop = 'BackdropSB',
	},
	[7] = {
		alpha = nil,
		object = MainTimerBarGrp,
		backdrop = 'BackdropSB',
	},
	[8] = {
		alpha = nil,
		object = UtilTimerBarGrp,
		backdrop = 'BackdropSB',
	},
	[9] = {
		alpha = true,
		object = MaelstromBar,
		statusbar = 150,
	},
	[10] = {
		alpha = false,
		object = DoomWindsTex,
		backdrop = 'BackdropSB',
	},
	[11] = {
		model = {
			[1] = StormstrikeCharges.Charge1,
			[2] = StormstrikeCharges.Charge2,
			[3] = StormstrikeCharges.Charge3,
		},
		object = StormstrikeChargeGrp,
		backdrop = "BackdropSB",
	},
}

SSA.MoveStringsEnh = {
	[1] = L["Move All Auras"],
	[2] = L["Move Top Auras"],
	[3] = L["Move Bottom Auras"],
	[4] = L["Move Left Auras"],
	[5] = L["Move Right Auras"],
	[6] = L["Move Buff Timer Bars"],
	[7] = L["Move Main Timer Bars"],
	[8] = L["Move Utility Timer Bars"],
	[9] = L["Move Maelstrom Bar"],
	[10] = L["Move Doom Winds"],
	[11] = L["Move Stormstrike Charges"],
}